#!/bin/sh

 #######################################################################
 # Copyright 2007-2009 Igor Daniloff.
 # 
 # Following source code is the property of Doctor Web, Ltd.
 # Use is subject to license terms.
 # The reproduction in any form without written permission of
 # Doctor Web, Ltd. and proper attribution is strongly prohibited.
 #######################################################################


#vars 

if [ -f /usr/xpg4/bin/awk ]; then
    AWK=/usr/xpg4/bin/awk
else
    AWK=awk
fi

if [ -f /usr/xpg4/bin/sed ]; then
    SED=/usr/xpg4/bin/sed
else
    SED=sed
fi

if [ -f /usr/xpg4/bin/grep ]; then
    GREP=/usr/xpg4/bin/grep
else
    GREP=grep
fi

if [ -f /usr/xpg4/bin/egrep ]; then
    EGREP=/usr/xpg4/bin/egrep
else
    EGREP=egrep
fi

DEBUG="no"
#ROOT_PATH=""
DEFAULT_BIN_PATH=/opt/drweb
DEFAULT_BIN_PATH=`echo "$DEFAULT_BIN_PATH" | $SED 's/^\///'`
SCRIPTS_DIR="$DEFAULT_BIN_PATH/maild/scripts"
DOC_DIR="$DEFAULT_BIN_PATH/doc/maild"
DEFAULT_VAR_PATH="$ROOT_PATH/var/drweb"
if [ "0" = "1" ]; then
   INIT_SCRIPT="$ROOT_PATH/usr/local/etc/rc.d/00.drweb-monitor.sh"
   SYS_LIB_DIR="/usr/local/lib"
   DW_LIB_DIR="$ROOT_PATH/usr/local/drweb/lib"
else
   INIT_SCRIPT="$ROOT_PATH/etc/init.d/drweb-monitor"
   SYS_LIB_DIR="/usr/local/lib"
   DW_LIB_DIR="$ROOT_PATH/opt/drweb/lib"
fi

#functions
######################################################################
dwd()
{
    if [ "$DEBUG" = "yes" ]; then
        echo $1
    fi
}
######################################################################
DwFindFullPath()
{
    if [ -x "./$0" ] ; then
        var=`echo $0`
        var=`echo "$var" | $SED 's/^\.\///'`
        SCRIPT_PATH=`pwd`/"$var"
        SCRIPT_PATH=`echo "$SCRIPT_PATH" | $SED 's/\"//g'`
    else
        SCRIPT_PATH="$0"
    fi
    SCRIPT_NAME=`echo "$SCRIPT_PATH" | $SED 's/^.*\/\([^\/]*\)$/\1/'`
    SCRIPT_PATH=`echo "$SCRIPT_PATH" | $SED 's/\/[^\/]*$//'`
    dwd "SCRIPT_PATH=$SCRIPT_PATH SCRIPT_NAME=$SCRIPT_NAME"
}

######################################################################
DwInstall()
{
(cd "$1"
    CHOWN=""
    ADD="no"
    for element in *; do
        if [ -d "$element" ]; then
            if [ ! -d "$2/$element" ]
            then
                if [ "$3" = "modif" -o "$3" = "update" ]
                then
                    mkdir -p "$2/$element" || exit 1
                    chown drweb:drweb "$2/$element" || exit 1
                    chmod 0775 "$2/$element" || exit 1
                    ADD="yes"
                    dwd "$element => $2/$element"
                fi
            fi
            DwInstall $element "$2/$element" $3 || exit 1
            if [ "$ADD" = "yes" ]
            then
                echo "$2/$element" >> "$DW_INSTALL_LOG"
            fi
        fi
        suffix=""
        if [ -f "$element"  -a "$element" != "$SCRIPT_NAME"  -a "$element" != "update.sh" ]; then
            ext=`echo "$element" | $SED 's/^.*\.\([^\.]*\)$/\1/'`
            if [ -f "$2/$element" ]; then
                if [ "$element" = "emails" -o "$ext" = "conf" -o "$ext" = "ini" -o "$ext" = "mmc" ]; then
                    if [ "$3" = "modif" ]
                    then
                        echo "File $2/$element exist: save it to $2/$element.drwebsave"
                        mv "$2/$element" "$2/$element.drwebsave" || exit 1
                    elif [ "$3" = "update" ]
                    then
                        echo "File $2/$element exist: save new version to $2/$element.new"
                        suffix=".new"
                    fi
                elif [ "$3" != "update" ]; then
                    echo "File $2/$element already exist: can not continue!"
                    exit 1
                fi
            fi
            if [ "$3" != "fake" ]
            then
                if [ ! -f "$2/$element$suffix" ]; then
                    echo "$2/$element$suffix" >> "$DW_INSTALL_LOG"
                    dwd "$element ($ext) => $2/$element$suffix"
                fi
                if [ -L "$element" ]; then
                    STAT_INFO=`ls -l "$element" | $SED "s/^.*-> \(.*\)$/\1/"`
                    (cd "$2"; if [ -f "$element$suffix" ]; then rm -f "$element$suffix"; fi; ln -s "$STAT_INFO" "$element$suffix"|| exit 1) || exit 1
                else
                    cp -p -R "$element" "$2/$element$suffix" || exit 1
                fi
                CHOWN="$CHOWN $2/$element$suffix"
            fi
        fi
    done
    if [ ! -z "$CHOWN" ]; then
        chown drweb:drweb $CHOWN || exit 1
    fi
)
}
######################################################################
DwStop()
{
    if [ ! -f $INIT_SCRIPT ]; then
        echo "Can not find init script $INIT_SCRIPT"
        exit 2
    fi
    
    $INIT_SCRIPT status
    if [ $? -eq 0 ]; then
        $INIT_SCRIPT stop
        if [ $? -ne 0 ]; then
            echo "Can not stop Dr.Web MailD"
            exit 3
        fi
    fi
}
######################################################################
DwReload()
{
    if [ -x $INIT_SCRIPT ]; then
        $INIT_SCRIPT status
        if [ $? -eq 0 ]; then
            $INIT_SCRIPT reload
            if [ $? -eq 0 ]; then
                echo "===> Dr.Web for Unix mail servers reloaded"
            else
                echo "" 
                echo "Automatic reload failed."
                echo "Please inspect Dr.Web for Unix mail servers and plugins configuration for default values."
                echo "Please start drweb-monitor manually."
            fi
        fi
    fi
}
######################################################################
DwRestart()
{
    if [ -x $INIT_SCRIPT ]; then
        $INIT_SCRIPT status
        if [ $? -eq 0 ]; then
            $INIT_SCRIPT restart
            if [ $? -eq 0 ]; then
                echo "===> Dr.Web for Unix mail servers restarted"
            else
                echo "" 
                echo "Automatic start failed."
                echo "Please inspect Dr.Web for Unix mail servers configuration for default values."
                echo "Please start drweb-monitor manually."
            fi
        fi
    elif [ -x "`which svcadm`" ] && svcs drweb-monitor >/dev/null 2>&1; then
        # Would not restart if disabled currently.
        svcadm restart drweb-monitor
    else
        echo ""
        echo "Dr.Web for Unix mail servers restart failed."
        echo "Neither SysV nor SVC service was found on your system."
    fi
}

######################################################################
DwBackup()
{
    if [ -f "$1" ] ; then

        if [ -f "$1.drwebsave" ] ; then

            CONF_DRWEBSAVE="yes"
            TIMESTAMP=`date "+%Y%m%d-%k_%M_%S"`
            echo ""
            echo "Info: $1.drwebsave exists. Saving $1 as $1.$TIMESTAMP"
            echo ""
            cp "$1" "$1.$TIMESTAMP"
            if [ $? -ne 0 ] ; then
                echo ""
                echo "Error: can't backup $1! Installation aborted."
                echo ""
                return 3
            fi
        fi
        if [ "$CONF_DRWEBSAVE" != "yes" ] ; then
            cp "$1" "$1.drwebsave"
            if [ $? -ne 0 ] ; then
                echo ""
                echo "Error: can't backup $1! Installation aborted."
                echo ""
                return 2
            else
                echo ""
                echo "Success: backup of $1 in $1.drwebsave."
                echo ""
            fi

        fi
    else
        echo ""
        echo "Error: can't find $1!"
        echo ""
        return 1
    fi
    return 0;
}
######################################################################
DwPostProc()
{
    chmod +s "$ROOT_PATH/$DEFAULT_BIN_PATH/drweb-inject"
}

######################################################################
DwInstallLibs()
{
:
#     for file in libstdc++.so.6 libgcc_s.so.1
#     do
#       if [ ! -f $SYS_LIB_DIR/$file ]; then
#           dwd "Make link from $DW_LIB_DIR/$file to $SYS_LIB_DIR/$file"
#           rm -f $SYS_LIB_DIR/$file >/dev/null 2>&1
#           ln -s $DW_LIB_DIR/$file $SYS_LIB_DIR/$file || exit 1
#       fi.
#     done
}
######################################################################
DwSetPluginName()
{
    OLD=`pwd`
    if [ ! -d "/$PLUGINS_DIR" ];then
        echo "Can not find plugin directory '/$PLUGINS_DIR'"
        exit 1
    fi
    cd "/$PLUGINS_DIR"
    for plugin in *; do
        PLUGIN_NAME=`echo "$plugin" | $SED 's/^lib\(.*\)\.so$/\1/'`
        dwd "PLUGIN_NAME = $PLUGIN_NAME"
        break
    done
    cd "$OLD"
    if [ "x$PLUGIN_NAME" = "x*" ]; then
        echo "Can not find any plugin library in /$PLUGINS_DIR"
        exit 1
    fi
}

DwConfigureMonitor()
{
    DEFAULT_CONF_PATH=/etc/drweb
    if test -f "$DEFAULT_CONF_PATH/monitor.conf" ; then
        RES=`cat $DEFAULT_CONF_PATH/monitor.conf | $EGREP "^User = root$"| head -n1`
        if test -z "$RES" ; then
	    MONITOR_CONF_SAVE="$DEFAULT_CONF_PATH/monitor.conf".drwebsave-`date +%Y%m%d_%H-%M-%S`
	    cp "$DEFAULT_CONF_PATH/monitor.conf" "$MONITOR_CONF_SAVE"
            $SED "s/User = drweb/User = root/" "$MONITOR_CONF_SAVE" > "$DEFAULT_CONF_PATH/monitor.conf"
            RES=`cat $DEFAULT_CONF_PATH/monitor.conf | $EGREP "^User = root$"| head -n1`
            if test -n "$RES" ; then
                echo "$DEFAULT_CONF_PATH/monitor.conf patched."
                if test -z $POSTINSTALL_FLAG; then
                    DwRestart
                fi
            else
                echo "$DEFAULT_CONF_PATH/monitor.conf not patched. Change User = drweb to User = root manually if needed."
            fi
        else
            echo "$DEFAULT_CONF_PATH/monitor.conf already patched."
        fi
    fi
}

DwConfigureMonitorRunAppList()
{
    DEFAULT_CONF_PATH=/etc/drweb
    if test -f "$DEFAULT_CONF_PATH/monitor.conf" ; then
        RES=`cat $DEFAULT_CONF_PATH/monitor.conf | $EGREP "^RunAppList = MAILDESK$"| head -n1`
        if test -z "$RES" ; then
	    MONITOR_CONF_SAVE="$DEFAULT_CONF_PATH/monitor.conf".drwebsave-`date +%Y%m%d_%H-%M-%S`
	    cp "$DEFAULT_CONF_PATH/monitor.conf" "$MONITOR_CONF_SAVE"
            $SED "s/RunAppList = MAILD/RunAppList = MAILDESK/" "$MONITOR_CONF_SAVE" > "$DEFAULT_CONF_PATH/monitor.conf"
            RES=`cat $DEFAULT_CONF_PATH/monitor.conf | $EGREP "^RunAppList = MAILDESK$"| head -n1`
            if test -n "$RES" ; then
                echo "$DEFAULT_CONF_PATH/monitor.conf patched."
                DwRestart
            else
                echo "$DEFAULT_CONF_PATH/monitor.conf not patched. Change 'RunAppList = MAILD' to 'RunAppList = MAILDESK' manually if needed."
            fi
        else
            echo "$DEFAULT_CONF_PATH/monitor.conf already patched."
        fi
    fi
}

############ PRE ##############

#root

id | $EGREP "uid=0\(" 1> /dev/null
if [ $? -ne 0 ]; then
    echo "You must be root to run the Dr.Web for Unix mail servers installer."
    exit 1
fi

#reset umask
umask 0002

cd "/$SCRIPTS_DIR"

DwFindFullPath


#vars
MTA_CONFIGURE="no"
DW_INSTALL_LOG="$DEFAULT_VAR_PATH/maild_EXIM_install.info"



if [ "x$MTA_CONFIG_PATH" != "x" ] ; then
    EXIM_CONF_DEFAULT="$MTA_CONFIG_PATH/exim.conf"
else
    EXIM_CONF_DEFAULT="/etc/exim/exim.conf"
fi

EXIM4_CF="$EXIM_CONF_DEFAULT /etc/exim/configure /etc/mail/exim.conf /etc/exim.conf /usr/local/etc/exim/exim.conf /usr/local/etc/exim/configure /usr/local/etc/mail/exim.conf /usr/local/etc/exim.conf \
/usr/exim/exim.conf /usr/exim/exim4.conf /usr/exim/configure /usr/local/exim/configure /etc/exim4/exim.conf /etc/exim4/configure /etc/exim4/exim4.conf.template /usr/local/etc/exim4/exim.conf /usr/local/etc/exim4/configure /usr/local/etc/exim4/exim4.conf.template"
EXIM4_CF_SPLIT="/etc/exim4/conf.d/ /usr/local/etc/exim4/conf.d/"

ALLREADY_PATCHED=0

allready_patched() {
    ap_config="$1"
    ap_config_dirs="main acl router transport retry rewrite auth"
    if [ -d $ap_config ] ; then
        for cdir in $ap_config_dirs ; do
            for cfile in ${ap_config}/${cdir}/* ; do
                cat ${cfile}
            done
        done | $GREP -q "^[^#]*drweb_router"
        [ $? -ne 0 ] && return 1
    elif [ -f $ap_config ]; then
        cat $ap_config | $GREP -q "^[^#]*drweb_router"
        [ $? -ne 0 ] && return 1
    else
        return 0
    fi
    ALLREADY_PATCHED=1
    return 0
}
find_binary() {
    if [ "x$EXIM_BIN" != "x" ] ; then
       fb_binary="$EXIM_BIN"
       if [ ! -f "$fb_binary" ] ; then
           fb_binary=""
       fi
    elif [ -f "/usr/sbin/exim" ] ; then
        fb_binary="/usr/sbin/exim"
    elif [ -f "/usr/sbin/exim4" ] ; then
        fb_binary="/usr/sbin/exim4"
    elif [ -f "/usr/bin/exim" ] ; then
        fb_binary="/usr/bin/exim"
    elif [ -f "/usr/bin/exim4" ] ; then
        fb_binary="/usr/bin/exim4"
    elif [ -f "/usr/local/bin/exim" ] ; then
        fb_binary="/usr/local/bin/exim"
    elif [ -f "/usr/local/bin/exim4" ] ; then
        fb_binary="/usr/local/bin/exim4"
    elif [ -f "/usr/local/sbin/exim" ] ; then
        fb_binary="/usr/local/sbin/exim"
    elif [ -f "/usr/local/sbin/exim4" ] ; then
        fb_binary="/usr/local/sbin/exim4"
    elif [ -f "/usr/exim/bin/exim" ] ; then
        fb_binary="/usr/exim/bin/exim"
    fi
    echo $fb_binary
}
backup_config() {
    if [ -d "$1" ] ; then
        return 0
    fi
    DwBackup $1
}
patch_exim4() {
    pe4_config="$1"
    allready_patched $pe4_config && return 1
    backup_config $pe4_config || return 2
    grep "trusted_users" "$pe4_config" >>/dev/null
    rc=$?
    if [ $rc -eq 0 ]; then
        cat "$pe4_config" | perl -e 'while(<>){if(/^trusted_users(.*)/){s/^trusted_users(.*)/###############################\n### MODIFIED BY MAILD-EXIM ####\n###############################\ntrusted_users$1 : drweb/;$modif=1;}print;}' > "$pe4_config.tmp1"
    elif [ $rc -eq 1 ]; then
        perl -e 'print "\n###############################\n### ADDED BY MAILD-EXIM #######\n###############################\ntrusted_users = drweb\n";' > "$pe4_config.tmp1"
        cat "$pe4_config" >> "$pe4_config.tmp1"
    fi
    cat "$pe4_config.tmp1" | $AWK '
{print}\
/begin routers/{\
print "# This router routes messages to drweb maild checking transport\n\
drweb_router:\n\
 driver = accept\n\
 condition = \"${if eq {$received_protocol}{drweb-scanned}{0}{1}}\"\n\
# check_local_user\n\
 retry_use_local_part\n\
 transport = drweb_transport"}\
/begin transports/{\
print "# This transport is used for checking messages in drweb maild\n\
drweb_transport:\n\
 driver = lmtp\n\
 socket = /var/drweb/ipc/.drweb_maild\n\
 batch_max = 100\n\
 timeout = 5m\n\
 user = drweb\n\
# headers_add = \"X-Maild-Checked: DrWEB for Exim\""}' > $pe4_config.tmp2
    rm -f $pe4_config.tmp1
    mv -f $pe4_config.tmp2 $pe4_config
    if [ $? -ne 0 ] ; then
        rm -f $pe4_config.tmp
        rm -f $pe4_config.tmp1
        return 1
    fi
    chmod 640 $pe4_config
    return 0
}
patch_exim4_split() {
    config_dir="$1"
    allready_patched $config_dir && return 1

    main_dir=${config_dir}/main
    main_file=${main_dir}/01_exim4-config_maild_drweb_user
    router_dir=${config_dir}/router
    router_file=${router_dir}/10_exim4-config_maild_drweb_router
    transport_dir=${config_dir}/transport
    transport_file=${transport_dir}/10_exim4-config_maild_drweb_transport

    if [ -f $main_file -a -s $main_file -o \
         -f $router_file -a -s $router_file -o \
         -f $transport_file -a -s $transport_file ] ; then
        return 0
    fi

    if [ -d $main_dir -a -d $router_dir -a -d $transport_dir ] ; then
        cat > $main_file <<EOF
# main/01_exim4-config_maild_drweb_user
#################################

trusted_users = drweb
EOF
        cat > $router_file <<EOF
# router/10_exim4-config_maild_drweb_router
#################################

# This router routes messages to drweb maild checking transport
drweb_router:
  driver = accept
  condition = "\${if eq {\$received_protocol}{drweb-scanned}{0}{1}}"
  retry_use_local_part
  transport = drweb_transport
EOF
        cat > $transport_file <<EOF
# transport/10_exim4-config_maild_drweb_transport
#################################

# This transport is used for checking messages in drweb maild
drweb_transport:
  driver = lmtp
  socket = /var/drweb/ipc/.drweb_maild
  batch_max = 100
  timeout = 5m
  user = drweb
  # headers_add = "X-Maild-Checked: DrWEB for Exim"
EOF
    fi

    if [ ! -s $main_file -o ! -s $router_file -o ! -s $transport_file ] ; then
        rm -f $main_file $router_file $transport_file
        return 3
    fi

    return 0
}
DwPrepareMta()
{
:
}
DwUpdateMTA()
{
:
}
DwInstallMtaEXIM()
{
    DwConfigureMonitor

    EXIM_BIN=`find_binary`
    if [ -z "$EXIM_BIN" ] ; then
        echo ""
        echo "Error: main executable file \"exim\" was not found!"
        return 1
    fi

    EXIM_VER=`$EXIM_BIN -bV | head -1 | cut -b 14`
    echo ""
    echo "Exim ver. $EXIM_VER.XX detected."
    
    OK=0
    FOUND=0
    if   [ "$EXIM_VER" != "3" ] ; then

        for EXIM_CONF in $EXIM4_CF ; do
            [ -f $EXIM_CONF ] && FOUND=1 && patch_exim4 "$EXIM_CONF" && OK=1 && break
            [ $ALLREADY_PATCHED -eq 1 ] && FOUND=1 && OK=1 && break
        done
        if [ $OK -eq 0 ] ; then
            FOUND=0
            for EXIM_CONF in $EXIM4_CF_SPLIT ; do
                [ -d $EXIM_CONF ] && FOUND=1 && patch_exim4_split "$EXIM_CONF" && OK=1  && break
                [ $ALLREADY_PATCHED -eq 1 ] && FOUND=1 && OK=1 && break
            done
        fi

    else
        echo "Error: unable to patch this version of Exim!"
        return 1
    fi
    if [ $FOUND -eq 0 ] ; then
        echo ""
        echo "Error: the Exim configuration file $EXIM_CONF_DEFAULT was not found!"
        echo "Info: you can specify the MTA_CONFIG_PATH environment variable."
        return 2
    elif [ $OK -eq 0 ] ; then
        echo ""
        echo "Error: the Exim configuration file $EXIM_CONF found, but can't been patched"
        return  3
    elif [ $ALLREADY_PATCHED -eq 1 ] ; then
        echo ""
        echo "Info: $EXIM_CONF contain drweb_router, skipping patching!"
        return 4
    else
        if [ $ALLREADY_PATCHED -eq 0 ] ; then
            (
                DW_ROOT_PATH=$ROOT_PATH;
                export DW_ROOT_PATH;
                cd "$ROOT_PATH/$SCRIPTS_DIR";
                echo "$EXIM_BIN" | perl -e '$EXIM_BIN=<>;chomp $EXIM_BIN;use dwconfig;my ($dw_root_path,$dw_bin_dir,$dw_etc_dir,$dw_conf_file) = DwInit();my $drweb_conf = DwLoadFile("$dw_etc_dir/$dw_conf_file");$drweb_conf = DwModify($drweb_conf,"Sender","Address",$EXIM_BIN,"string");DwSaveFile("$dw_etc_dir/$dw_conf_file",$drweb_conf);'
                );
        fi
        echo ""
        echo "$EXIM_CONF successfull patched"
        echo ""
    fi

    MTA_CONFIGURE="yes"
}



############ START ##############

if [ "x$1" = "xupdate" ] ; then
    echo
    echo ------------------------------------------------------------
    echo     Welcome to the Dr.Web InstallShield Wizard.
    echo
    echo The InstallShield Wizard will update configuration of EXIM.
    echo
    while true ; do
        echo "Do you wish to continue?"
        read yesno
        case "$yesno" in
            y | yes | Y | Yes | YES)
            break
            ;;
            n | no | N | No | NO)
            echo "Try later. Bye!"
            exit 1
            ;;
            *)
            echo Please enter yes or no.
            ;;
        esac
    done
    DWPrepareMta
    DwUpdateMTA
fi


DwPrepareMta

#configure MTA
if  [ "x$SKIP_MTA_CONFIG" != "xyes" ]; then
    echo
    echo ------------------------------------------------------------
    echo     Welcome to the Dr.Web InstallShield Wizard.
    echo
    echo The InstallShield Wizard will configure EXIM.
    echo
    while true ; do
        echo "Perform MTA configuration?"
	echo "Please enter yes or no."
        read yesno
        case "$yesno" in
            y | yes | Y | Yes | YES)
            break
            ;;
            n | no | N | No | NO)
            echo "Try later. Bye!"
            exit 1
            ;;
            *)
            echo Please enter yes or no.
            ;;
        esac
    done
    DwInstallMtaEXIM
fi

if [ "EXIM" != "MAILDESK" ]; then
  if [ "$MTA_CONFIGURE" = "no" ]
  then
    echo "Please, refer to documentation on EXIM adjustment residing in $ROOT_PATH/$DOC_DIR directory."
  else
    echo "Please, restart EXIM manually."
  fi
fi

